/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package exociel;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.Vector;

import xml.*;

/**
 * Carte du ciel des toiles autour desquelles orbitent des plantes dtectes
 *
 * Dimensions: 500x500
 *
 * Paramtres:
 * etoiles - nom du fichier XML avec les noms et coordonnes des toiles
 *           arbre :
 *           ETOILES
 *              +-- ETOILE @nom
 *                    +-- AD
 *                    +-- DEC
 *                    +-- DIST  (distance en parsec)
 *                    +-- NP - nombre de plantes
 * nord - nom du fichier d'image GIF avec la carte du ciel de l'hmisphre Nord
 * sud - nom du fichier d'image GIF avec la carte du ciel de l'hmisphre Sud
 */
public class ExoCiel extends Applet implements ItemListener {
    
    private static final int espace = 15;
    private static final String baseChemin = "systeme.php?etoile=";
    private int dimx, dimy;
    private Image imageCarteNord, imageCarteSud;
    private String[] tnomsNord;
    private String[] tnomsSud;
    private int[] txNord;
    private int[] txSud;
    private int[] tyNord;
    private int[] tySud;
    private double[] tdistNord;
    private double[] tdistSud;
    private int[] tnpNord;
    private int[] tnpSud;
    private ImageCanvas ic;
    private CheckboxGroup boites;
    private Checkbox chkNord, chkSud, chkSel;
    
    public void init() {
        setLayout(new BorderLayout());
        setBackground(new Color(0, 34, 62));
        setForeground(Color.white);
        String fichierXML = getParameter("etoiles");
        String fichierCarteNord = getParameter("nord");
        String fichierCarteSud = getParameter("sud");
        imageCarteNord = getImage(getDocumentBase(), fichierCarteNord);
        imageCarteSud = getImage(getDocumentBase(), fichierCarteSud);
        if (imageCarteNord != null) {
            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(imageCarteNord, 0);
            tracker.addImage(imageCarteSud, 0);
            try {
                tracker.waitForAll();
            } catch (InterruptedException ex) {
                System.err.println("InterruptedException: " + ex.getMessage());
            }
            dimx = imageCarteNord.getWidth(this);
            dimy = imageCarteNord.getHeight(this);
        }
        lectureXML(fichierXML);
        ic = new ImageCanvas(imageCarteNord, dimx, dimy, tnomsNord, tdistNord, tnpNord, txNord, tyNord, this);
        add(ic, BorderLayout.CENTER);
        boites = new CheckboxGroup();
        Panel bpane = new Panel();
        chkNord = new Checkbox(Messages.get("hemisphere_nord"), boites, true);
        chkNord.addItemListener(this);
        bpane.add(chkNord);
        chkSud = new Checkbox(Messages.get("hemisphere_sud"), boites, false);
        chkSud.addItemListener(this);
        bpane.add(chkSud);
        chkSel = chkNord;
        add(bpane, BorderLayout.SOUTH);
        
        // pour viter un bug de Tiger, on doit rpter les couleurs pour chaque composant
        modifCouleurs(chkNord);
        modifCouleurs(chkSud);
    }
    
    protected void modifCouleurs(Component c) {
        c.setBackground(new Color(0, 34, 62));
        c.setForeground(Color.white);
    }
    
    protected void lectureXML(String fichierXML) {
        XMLTree tree = new XMLTree(null, "", null, "");
        URL hurle;
        try {
            if (fichierXML.startsWith("http://"))
                hurle = new URL(fichierXML);
            else
                hurle = new URL(getDocumentBase(), fichierXML);
        } catch (MalformedURLException ex) {
            System.err.println("MalformedURLException: " + ex.getMessage());
            return;
        }
        Parser_XML p = new Parser_XML(hurle, System.out, tree);
        p.parse(0);
        if (!"ETOILES".equalsIgnoreCase(tree.tag)) {
            System.err.println("Mauvais fichier XML !");
            return;
        }
        XMLTree etoile;
        String nom, sad, sdec, sdist, snp;
        Vector vnomsNord = new Vector();
        Vector vnomsSud = new Vector();
        Vector vadNord = new Vector();
        Vector vadSud = new Vector();
        Vector vdecNord = new Vector();
        Vector vdecSud = new Vector();
        Vector vdistNord = new Vector();
        Vector vdistSud = new Vector();
        Vector vnpNord = new Vector();
        Vector vnpSud = new Vector();
        for (etoile=tree.first_child; etoile!=null; etoile=etoile.next_brother) {
            nom = etoile.getAttVal("nom");
            sad = etoile.getNode("AD").val;
            sdec = etoile.getNode("DEC").val;
            sdist = etoile.getNode("DIST").val;
            snp = etoile.getNode("NP").val;
            if (!sdec.trim().startsWith("-")) {
                vnomsNord.addElement(nom);
                vadNord.addElement(Double.valueOf(sad));
                vdecNord.addElement(Double.valueOf(sdec));
                vdistNord.addElement(sdist);
                vnpNord.addElement(snp);
            } else {
                vnomsSud.addElement(nom);
                vadSud.addElement(Double.valueOf(sad));
                vdecSud.addElement(Double.valueOf(sdec));
                vdistSud.addElement(sdist);
                vnpSud.addElement(snp);
            }
        }
        
        tnomsNord = new String[vnomsNord.size()];
        tnomsSud = new String[vnomsSud.size()];
        for (int i=0; i<vnomsNord.size(); i++)
            tnomsNord[i] = (String)vnomsNord.elementAt(i);
        for (int i=0; i<vnomsSud.size(); i++)
            tnomsSud[i] = (String)vnomsSud.elementAt(i);
        
        tdistNord = versDoubles(vdistNord);
        tdistSud = versDoubles(vdistSud);
        
        tnpNord = versEntiers(vnpNord);
        tnpSud = versEntiers(vnpSud);
        
        txNord = new int[vadNord.size()];
        txSud = new int[vadSud.size()];
        tyNord = new int[vadNord.size()];
        tySud = new int[vadSud.size()];
        double ad, adaff, dec;
        double rdec;
        int r = dimx/2 - espace;
        int cx = dimx/2;
        int cy = dimy/2;
        for (int i=0; i<vadNord.size(); i++) {
            ad = ((Double)vadNord.elementAt(i)).doubleValue();
            adaff = (360 - ad - 90) * Math.PI/180;
            dec = ((Double)vdecNord.elementAt(i)).doubleValue();
            rdec = ((90 - dec) / 90) * r;
            txNord[i] = (int)Math.round(cx + rdec * Math.cos(adaff));
            tyNord[i] = (int)Math.round(cy - rdec * Math.sin(adaff));
        }
        for (int i=0; i<vadSud.size(); i++) {
            ad = ((Double)vadSud.elementAt(i)).doubleValue();
            adaff = (ad + 90) * Math.PI/180;
            dec = ((Double)vdecSud.elementAt(i)).doubleValue();
            rdec = ((90 + dec) / 90) * r;
            txSud[i] = (int)Math.round(cx + rdec * Math.cos(adaff));
            tySud[i] = (int)Math.round(cy - rdec * Math.sin(adaff));
        }
    }
    
    public void ouvrirEtoile(String nom) {
        try {
            URL hurle = new URL(getDocumentBase(), baseChemin + URLEncoder.encode(nom));
            getAppletContext().showDocument(hurle, "_blank");
        } catch (MalformedURLException ex) {
            System.err.println("MalformedURLException: " + ex.getMessage());
        }
    }
    
    // slection de l'hmisphre (Nord ou Sud)
    protected void selectionH() {
        if (chkSel == chkNord)
            ic.changerDonnees(imageCarteNord, tnomsNord, tdistNord, tnpNord, txNord, tyNord);
        else
            ic.changerDonnees(imageCarteSud, tnomsSud, tdistSud, tnpSud, txSud, tySud);
    }
    
    public void itemStateChanged(ItemEvent e) {
        Checkbox chk = boites.getSelectedCheckbox();
        if (chk != chkSel) {
            chkSel = chk;
            selectionH();
        }
    }
    
    // Vector de String -> tableau de double
    private static double[] versDoubles(Vector v) {
        if (v == null || v.size() == 0)
            return(null);
        double[] t = new double[v.size()];
        for (int i=0; i<v.size(); i++) {
            if ("".equals((String)v.elementAt(i)))
                t[i] = 0;
            else {
                try {
                    t[i] = (new Double((String)v.elementAt(i))).doubleValue();
                } catch (NumberFormatException ex) {
                    System.err.println(((String)v.elementAt(i)) + " -> NumberFormatException: " + ex.getMessage());
                    t[i] = 0;
                }
            }
        }
        return(t);
    }

    // Vector de String -> tableau de int
    private static int[] versEntiers(Vector v) {
        if (v == null || v.size() == 0)
            return(null);
        int[] t = new int[v.size()];
        for (int i=0; i<v.size(); i++) {
            try {
                t[i] = (new Integer((String)v.elementAt(i))).intValue();
            } catch (NumberFormatException ex) {
                System.err.println(((String)v.elementAt(i)) + " -> NumberFormatException: " + ex.getMessage());
                t[i] = 0;
            }
        }
        return(t);
    }
}
