/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package exociel;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

/**
 * Zone d'affichage de la carte
 */
public class ImageCanvas extends Canvas implements MouseListener, MouseMotionListener {
    private Dimension tailleImage;
    private int dimx, dimy;
    private Image imageCarte = null;
    private int indEtoileSel;
    private String[] tnoms;
    private double[] tdist;
    private int[] tnp;
    private int[] tx;
    private int[] ty;
    private ExoCiel app;
    private Image offImage = null;
    
    public ImageCanvas(Image image, int dimx, int dimy, String[] tnoms, double[] tdist, int[] tnp,
            int[] tx, int[] ty, ExoCiel app) {
        this.imageCarte = image;
        this.dimx = dimx;
        this.dimy = dimy;
        this.tnoms = tnoms;
        this.tdist = tdist;
        this.tnp = tnp;
        this.tx = tx;
        this.ty = ty;
        this.app = app;
        tailleImage = new Dimension(dimx, dimy);
        offImage = app.createImage(dimx, dimy);
        indEtoileSel = -1;
        if (tx != null && ty != null) {
            addMouseListener(this);
            addMouseMotionListener(this);
        }
    }
    
    public void changerDonnees(Image image, String[] tnoms, double[] tdist, int[] tnp, int[] tx, int[] ty) {
        this.imageCarte = image;
        this.tnoms = tnoms;
        this.tdist = tdist;
        this.tnp = tnp;
        this.tx = tx;
        this.ty = ty;
        indEtoileSel = -1;
        repaint();
    }
    
    public Dimension getPreferredSize() {
        return tailleImage;
    }
    
    public Dimension getMinimumSize() {
        return tailleImage;
    }
    
    public void paint(Graphics g) {
        if (offImage != null) {
            Graphics offg = offImage.getGraphics();
            if (imageCarte != null)
                offg.drawImage(imageCarte, 0, 0, this);
            offg.drawRect(0, 0, dimx-1, dimy-1);
            dessinerEtoiles(offg);
            
            if (indEtoileSel != -1)
                dessinerSelection(offg);
            
            g.drawImage(offImage, 0, 0, this);
        }
    }
    
    protected void dessinerEtoiles(Graphics g) {
        g.setColor(Color.yellow);
        for (int i=0; i<tx.length; i++) {
            g.drawLine(tx[i] - 5, ty[i], tx[i] + 5, ty[i]);
            g.drawLine(tx[i], ty[i] - 5, tx[i], ty[i] + 5);
        }
        g.setColor(Color.black);
    }
    
    protected void dessinerSelection(Graphics g) {
        int xe = tx[indEtoileSel];
        int ye = ty[indEtoileSel];
        g.setColor(Color.red);
        g.drawLine(xe - 5, ye, xe + 5, ye);
        g.drawLine(xe, ye - 5, xe, ye + 5);
        String nom = tnoms[indEtoileSel];
        String dist;
        if (tdist[indEtoileSel] == 0)
            dist = Messages.get("distance") + ": ?";
        else
            dist = Messages.get("distance") + ": " + tdist[indEtoileSel] + " pc";
        String snp;
        if (tnp[indEtoileSel] == 0)
            snp = Messages.get("nb_planetes") + ": ?";
        else
            snp = Messages.get("nb_planetes") + ": " + tnp[indEtoileSel];
        int w = g.getFontMetrics().stringWidth(nom);
        int w2 = g.getFontMetrics().stringWidth(dist);
        int w3 = g.getFontMetrics().stringWidth(snp);
        if (w2 > w)
            w = w2;
        if (w3 > w)
            w = w3;
        if (xe + w + 6 > dimx)
            xe = dimx - w - 6;
        if (ye + 42 > dimy)
            ye = dimy - 42;
        g.setColor(new Color(255, 255, 203));
        g.fillRoundRect(xe+2, ye+2, w+4, 42, 6, 6);
        g.setColor(Color.black);
        g.drawString(nom, xe+4, ye+4+12);
        g.drawString(dist, xe+4, ye+4+24);
        g.drawString(snp, xe+4, ye+4+36);
    }
    
    public void update(Graphics g) {
        paint(g);
    }
    
    protected int chercherEtoile(int xs, int ys) {
        double distmin = 1E10;
        int indice = -1;
        int xe, ye;
        int xemxs, yemys;
        double tempdistmin;
        for (int i=0; i<tnoms.length; i++) {
            xe = tx[i];
            ye = ty[i];
            xemxs = xe-xs;
            yemys = ye-ys;
            tempdistmin = xemxs*xemxs + yemys*yemys;
            if (tempdistmin < distmin) {
                distmin = tempdistmin;
                indice = i;
            }
        }
        if (distmin < 20*20*2)
            return(indice);
        else
            return(-1);
    }
    
    public void mouseClicked(MouseEvent e) {
        int indice = chercherEtoile(e.getX(), e.getY());
        if (indice != -1)
            app.ouvrirEtoile(tnoms[indice]);
    }
    
    public void mousePressed(MouseEvent e) {
    }
    
    public void mouseReleased(MouseEvent e) {
    }
    
    public void mouseEntered(MouseEvent e) {
    }
    
    public void mouseExited(MouseEvent e) {
    }
    
    public void mouseDragged(MouseEvent e) {
    }
    
    public void mouseMoved(MouseEvent e) {
        int indice = chercherEtoile(e.getX(), e.getY());
        if (indice != indEtoileSel) {
            if (indEtoileSel == -1)
                setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            else if (indice == -1)
                setCursor(Cursor.getDefaultCursor());
            indEtoileSel = indice;
            repaint();
        }
    }
}
